import { mplCore } from '@metaplex-foundation/mpl-core';
import { createNoopSigner, generateSigner, signerIdentity, signerPayer, } from '@metaplex-foundation/umi';
import { createUmi } from '@metaplex-foundation/umi-bundle-defaults';
import { existsSync, lstatSync } from 'node:fs';
import { createSignerFromFile } from './FileSigner.js';
import { createSignerFromLedgerPath } from './LedgerSigner.js';
import { readJsonSync } from './file.js';
import initStorageProvider from './uploader/initStorageProvider.js';
import { DUMMY_UMI } from './util.js';
import { mplToolbox } from '@metaplex-foundation/mpl-toolbox';
import { mplTokenMetadata } from '@metaplex-foundation/mpl-token-metadata';
export const DEFAULT_CONFIG = {
    commitment: 'confirmed',
    keypair: '~/.config/solana/id.json',
    rpcUrl: 'https://api.devnet.solana.com',
};
export const CONFIG_KEYS = [
    'keypair',
    'payer',
    'rpcUrl',
    'commitment',
    'storage',
    'wallets',
    'rpcs',
    'explorer',
];
export const getDefaultConfigPath = (prefix) => `${prefix}/config.json`;
export const readConfig = (path) => {
    if (!existsSync(path)) {
        return {};
    }
    if (lstatSync(path).isDirectory()) {
        throw new Error(`Config file path is a directory, expected a file: ${path}`);
    }
    const configJson = readJsonSync(path);
    const filteredConfig = {};
    for (const key of CONFIG_KEYS) {
        filteredConfig[key] = configJson[key];
    }
    return filteredConfig;
};
export const createSignerFromPath = async (path) => {
    if (path.startsWith('usb://ledger')) {
        return createSignerFromLedgerPath(path);
    }
    if (existsSync(path)) {
        return createSignerFromFile(path);
    }
    // TODO move this warning to a better place
    console.log('[warning]: No keypair specified, using temporary noop-signer');
    // create no-op signer if no key is specified
    const kp = generateSigner(DUMMY_UMI);
    return createNoopSigner(kp.publicKey);
};
export function consolidateConfigs(...configs) {
    return configs.reduce((acc, config) => {
        for (const key in config) {
            if (config.hasOwnProperty(key)) {
                const value = config[key];
                if (value !== undefined) {
                    acc[key] = value;
                }
            }
        }
        return acc;
    }, {});
}
export const createContext = async (configPath, overrides) => {
    const config = consolidateConfigs(DEFAULT_CONFIG, readConfig(configPath), overrides);
    const signer = await createSignerFromPath(config.keypair);
    const payer = config.payer ? await createSignerFromPath(config.payer) : signer;
    const umi = createUmi(config.rpcUrl, {
        commitment: config.commitment,
    });
    umi.use(signerIdentity(signer))
        .use(signerPayer(payer))
        .use(mplCore())
        .use(mplTokenMetadata())
        .use(mplToolbox());
    if (config.storage) {
        const storageProvider = await initStorageProvider(umi, config);
        storageProvider && umi.use(storageProvider);
    }
    return {
        commitment: config.commitment,
        payer,
        rpcUrl: config.rpcUrl,
        signer,
        umi,
        explorer: config.explorer || 'solanaExplorer',
    };
};
